################
#
#  infilt.awk
#
#   awk script to convert infilt.conf  (and output of rpc2filt.awk)
#	to a shell script to change kernel configuration variables
#

BEGIN {
	# set some defaults
	enable = 1
	debug = 1
	clear_options = 1
	kill_options = 1
	udp_dport_min = 1024
	tcp_dport_min = 1024

	tcp_window_max = 1024
	tcp_mss_max = 512

	# create tables -- use bogus 0 entry to do this
	tcp_no_window_max[0] = 1
	icmp_type_zap[0] = 1
	udp_dport_ok[0] = 1
	udp_dport_zap[0] = 1
	tcp_dport_ok[0] = 1
	tcp_dport_zap[0] = 1

	# no error state
	error = 0
	error_line = ""
}


$1 == "enable" {
	enable = $2
	next
}

$1 == "debug" {
	debug = $2
	next
}

$1 == "clear_options" {
	clear_options = $2
	next
}

$1 == "kill_options" {
	kill_options = $2
	next
}


$1 == "udp_dport_min" {
	udp_dport_min = $2
	next
}

$1 == "tcp_dport_min" {
	tcp_dport_min = $2
	next
}

$1 == "tcp_window_max" {
	tcp_window_max = $2
	next
}

$1 == "tcp_mss_max" {
	tcp_mss_max = $2
	next
}


$1 == "tcp_no_window_max" {
	tcp_no_window_max[$2] = 1
	++ num_tcp_no_window_max
	if ( num_tcp_no_window_max > 98 ) {
		error = 1
		print "echo '*** ERROR: Too many tcp_no_window_max commands' " 
	}
	next
}

$1 == "icmp_type_zap" {
	icmp_type_zap[$2] = 1
	++ num_icmp_type_zap
	if ( num_icmp_type_zap > 98 ) {
		error = 1
		print "echo '*** ERROR: Too many icmp_type_zap commands' " 
	}
	next
}

$1 == "udp_dport_ok" {
	udp_dport_ok[$2] = 1
	++ num_udp_dport_ok
	if ( num_udp_dport_ok > 98 ) {
		error = 1
		print "echo '*** ERROR: Too many udp_dport_ok commands' " 
	}
	next
}

$1 == "udp_dport_zap" {
	udp_dport_zap[$2] = 1
	++ num_udp_dport_zap
	if ( num_udp_dport_zap > 98 ) {
		error = 1
		print "echo '*** ERROR: Too many udp_dport_zap commands' " 
	}
	next
}

$1 == "tcp_dport_ok" {
	tcp_dport_ok[$2] = 1
	++ num_tcp_dport_ok
	if ( num_tcp_dport_ok > 98 ) {
		error = 1
		print "echo '*** ERROR: Too many tcp_dport_ok commands' " 
	}
	next
}

$1 == "tcp_dport_zap" {
	tcp_dport_zap[$2] = 1
	++ num_tcp_dport_zap
	if ( num_tcp_dport_zap > 98 ) {
		error = 1
		print "echo '*** ERROR: Too many tcp_dport_zap commands' " 
	}
	next
}

/^[#]/ {
	# ignore comments
	next
}

/^[ \t]*$/ {
	# ignore empty lines
	next
}

{
	# this is executed if a "next" was not found above
	error = 1
	print "echo '*** ERROR IN THIS LINE: " $0 " ' "
}

END {

  if ( error ) {
	print "exit 1"
  } else {
	print "adb -k -w /vmunix /dev/mem <<END "

	print "infilt_enable  /W 0t" enable
	print "infilt_debug  /W 0t" debug
	print "infilt_clear_options  /W 0t" clear_options
	print "infilt_kill_options  /W 0t" kill_options
	print "infilt_udp_dport_min  /W 0t" udp_dport_min
	print "infilt_tcp_dport_min  /W 0t" tcp_dport_min
	print "infilt_tcp_window_max  /W 0t" tcp_window_max
	print "infilt_tcp_mss_max  /W 0t" tcp_mss_max

	x = 0
	for ( i in tcp_no_window_max ) {
	   if ( i != 0 ) {
		print "infilt_tcp_no_window_max+0t" x "  /W 0t" i
		x += 4
	   }
	}
	print "infilt_icmp_type_zap+0t" x "  /W 0"

	x = 0
	for ( i in icmp_type_zap ) {
	   if ( i != 0 ) {
		print "infilt_icmp_type_zap+0t" x "  /W 0t" i
		x += 4
	   }
	}
	print "infilt_icmp_type_zap+0t" x "  /W 0"

	x = 0
	for ( i in udp_dport_ok ) {
	   if ( i != 0 ) {
		print "infilt_udp_dport_ok+0t" x "  /W 0t" i
		x += 4
	   }
	}
	print "infilt_udp_dport_ok+0t" x "  /W 0"

	x = 0
	for ( i in udp_dport_zap ) {
	   if ( i != 0 ) {
		print "infilt_udp_dport_zap+0t" x "  /W 0t" i
		x += 4
	   }
	}
	print "infilt_udp_dport_zap+0t" x "  /W 0"

	x = 0
	for ( i in tcp_dport_ok ) {
	   if ( i != 0 ) {
		print "infilt_tcp_dport_ok+0t" x "  /W 0t" i
		x += 4
	   }
	}
	print "infilt_tcp_dport_ok+0t" x "  /W 0"

	x = 0
	for ( i in tcp_dport_zap ) {
	   if ( i != 0 ) {
		print "infilt_tcp_dport_zap+0t" x "  /W 0t" i
		x += 4
	   }
	}
	print "infilt_tcp_dport_zap+0t" x "  /W 0"

	print "END"

  }


}
